#ifndef DEBTAGS_OPTIONS_H
#define DEBTAGS_OPTIONS_H

/*
 * Commandline parser for tagcoll
 *
 * Copyright (C) 2003,2004,2005,2006  Enrico Zini
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <tagcoll/Commandline.h>

namespace Tagcoll {
namespace commandline {

struct DebtagsOptions : public MainParser<CommandParser>
{
	struct HelpGroup : public OptionGroup
	{
		BoolOption* help;
		BoolOption* version;

		HelpGroup()
		{
			add(help = new BoolOption("help", 'h', "help"));
			add(version = new BoolOption("version", 'V', "version"));
			help->shortNames.push_back('?');
			help->description = "print an help message and exit";
			version->description = "print the program version and exit";
			description = "Help options";
		}
		~HelpGroup()
		{
			delete help; delete version;
		}
	} helpGroup;

	struct OutputGroup : public OptionGroup
	{
		BoolOption* group;
		BoolOption* quiet;
		BoolOption* names;
		BoolOption* verbose;
		BoolOption* debug;
		
		OutputGroup()
		{
			add(group		= new BoolOption("group", 0, "group"));
			add(quiet		= new BoolOption("quiet", 'q', "quiet"));
			add(names		= new BoolOption("names", 0, "names"));
			add(verbose		= new BoolOption("verbose", 'v', "verbose"));
			add(debug		= new BoolOption("debug", 0, "debug"));
			group->description = "group items with the same tagset in the output collection";
			group->longNames.push_back("group-items");
			quiet->description = "do not write anything to standard output";
			names->description = "output only the names of the packages";
			verbose->description = "enable verbose output";
			debug->description = "enable debugging output (including verbose output)";
			description = "Options controlling transformations of output data";
		}
		~OutputGroup()
		{
			delete group;
			delete quiet;
			delete names;
			delete verbose;
			delete debug;
		}
	} outputGroup;

	struct MatchGroup : public OptionGroup
	{
		BoolOption* invert;

		MatchGroup()
		{
			add(invert = new BoolOption("invert", 'i', "invert"));
			invert->description = "invert the match, selecting non-matching items";
			description = "Options controlling matching of packages";
		}
		~MatchGroup()
		{
			delete invert;
		}
	} matchGroup;


	struct Generic : public OptionParser
	{
		Generic(DebtagsOptions* cp) : OptionParser("")
		{
			add(&cp->helpGroup);
		}
	} generic;
	struct Help : public OptionParser
	{
		Help(DebtagsOptions* cp) : OptionParser("help")
		{
			usage = "[command]";
			description = "print help informations";
		}
	} help;
	struct Update : public OptionParser
	{
		BoolOption* local;

		Update(DebtagsOptions* cp) : OptionParser("update")
		{
			add(local = new BoolOption("local", 0, "local"));
			local->usage = "";
			local->description = "do not download files when performing an update";
			usage = "";
			description = "updates the package tag database (requires root)";
			longDescription =
				"Collect package tag data from the sources listed in "
				"/etc/debtags/sources.list, then regenerate the debtags "
				"tag database and main index.\n"
				"It needs to be run as root";
			add(&cp->helpGroup);
		}
	} update;
	struct Selfcheck : public OptionParser
	{
		Selfcheck(DebtagsOptions* cp) : OptionParser("selfcheck")
		{
			usage = "";
			description = "perform a series of internal self checks using the current tag data";
			add(&cp->helpGroup);
		}
	} selfcheck;
	struct Check : public OptionParser
	{
		Check(DebtagsOptions* cp) : OptionParser("check")
		{
			usage = "<file>";
			description =
				"check that all the tags in the given tagged collection are present "
				"in the tag vocabulary.  Checks the main database if no file is "
				"specified";
			add(&cp->helpGroup);
		}
	} check;
	struct Tagcat : public OptionParser
	{
		Tagcat(DebtagsOptions* cp) : OptionParser("tagcat")
		{
			usage = "";
			description = "output the tag vocabulary";
			add(&cp->helpGroup);
		}
	} tagcat;
	struct Tagshow : public OptionParser
	{
		Tagshow(DebtagsOptions* cp) : OptionParser("tagshow")
		{
			usage = "<tag>";
			description = "show the vocabulary informations about a tag";
			add(&cp->helpGroup);
		}
	} tagshow;
	struct Tagsearch : public OptionParser
	{
		Tagsearch(DebtagsOptions* cp) : OptionParser("tagsearch")
		{
			usage = "<string [string [string ...]]>";
			description = "show a summary of all tags whose data contains the given strings";
			add(&cp->helpGroup);
		}
	} tagsearch;
	struct Show : public OptionParser
	{
		Show(DebtagsOptions* cp) : OptionParser("show")
		{
			usage = "<pkg>";
			description = "show informations about a package, like apt-cache show does, but "
						  "adding the tag informations from the debtags index";
			add(&cp->helpGroup);
		}
	} show;
	struct Related : public OptionParser
	{
		IntOption* distance;
		
		Related(DebtagsOptions* cp) : OptionParser("related")
		{
			add(distance = new IntOption("distance", 'd', "distance"));
			distance->description = "set the maximum distance to use for the \"related\" command (defaults to 0)";
			
			usage = "<pkg1[,pkg2[,pkg3...]]>";
			description = "show packages related to the given one(s)";
			longDescription =
				"Output a list of the packages that are related to the given package or list of packages.  "
				"If more than one package are to be specified, separate them with commas.\n"
				"The --distance option can be used to control how closely related the output "
				"packages should be from the package(s) specified.";
			examples = "debtags related mutt,mozilla-browser";

			add(&cp->helpGroup);
		}
		~Related()
		{
			delete distance;
		}
	} related;
	struct Cat : public OptionParser
	{
		Cat(DebtagsOptions* cp) : OptionParser("cat")
		{
			usage = "";
			description = "output the full package tag database";
			add(&cp->outputGroup);
			add(&cp->helpGroup);
		}
	} cat;
	struct Dumpavail : public OptionParser
	{
		Dumpavail(DebtagsOptions* cp) : OptionParser("dumpavail")
		{
			usage = "[tag expression]";
			description = "output the full package database";
			add(&cp->matchGroup);
			add(&cp->helpGroup);
		}
	} dumpavail;
	struct Search : public OptionParser
	{
		Search(DebtagsOptions* cp) : OptionParser("search")
		{
			usage = "<tag expression>";
			description =
				"output the names and descriptions of the packages that match"
				"the given tag expression";
			add(&cp->matchGroup);
			add(&cp->helpGroup);
		}
	} search;
	struct Grep : public OptionParser
	{
		Grep(DebtagsOptions* cp) : OptionParser("grep")
		{
			usage = "<tag expression>";
			description =
				"output the lines of the full package tag database that match"
				"the given tag expression";
			add(&cp->matchGroup);
			add(&cp->outputGroup);
			add(&cp->helpGroup);
		}
	} grep;
	struct Install : public OptionParser
	{
		Install(DebtagsOptions* cp) : OptionParser("install")
		{
			usage = "<tag expression>";
			description =
				"apt-get install the packages that match the given tag expression";
			longDescription =
				"Invokes apt-get install with the names of the packages matched "
				"by the given tag expression.  If you want to see what packages "
				"would be installed you can use debtags search, as "
				"debtags install just calls apt-get install on all "
				"the results of an equivalent debtags search.  Please note "
				"that debtags install is just a prototype feature useful "
				"for experimenting in some environments like Custom Debian "
				"Distributions.  For this reason it is suggested that you "
				"use debtags just as a way to find packages, and "
				"proper package managers as the way to install them";
			add(&cp->matchGroup);
			add(&cp->helpGroup);
		}
	} install;
	struct Diff : public OptionParser
	{
		Diff(DebtagsOptions* cp) : OptionParser("diff")
		{
			usage = "[filename]";
			description =
				"create a tag patch between the current tag database and the tag"
				"collection [filename].  Standard input is used if filename is not specified";
			add(&cp->helpGroup);

			aliases.push_back("mkpatch");
		}
	} diff;
	struct Maintainers : public OptionParser
	{
		Maintainers(DebtagsOptions* cp) : OptionParser("maintainers")
		{
			usage = "";
			description =
				"create a tagged collection of maintainers and the tags of the"
				"packages they maintain";
			add(&cp->outputGroup);
			add(&cp->helpGroup);
		}
	} maintainers;
	struct Tag : public OptionParser
	{
		Tag(DebtagsOptions* cp) : OptionParser("tag")
		{
			usage = "{add|rm|ls} <package> [tags...]";
			description = "view and edit the tags for a package";
			longDescription =
				"General manipulation of tags, useful for automation in scripts.\n"
				"It can be used in three ways:\n"
				"tag add <package> <tags...> will add the tags to the given package\n"
			    "tag rm <package> <tags...> will remove the tags from the given package\n"
				"tag ls <package> will output the names of the tags of the given package";
			add(&cp->helpGroup);
		}
	} tag;
	struct Submit : public OptionParser
	{
		Submit(DebtagsOptions* cp) : OptionParser("submit")
		{
			usage = "[patch]";
			description =
				"mail the given patch file to the central tag repository."
				"If [patch] is omitted, mail the local tag modifications.";
			add(&cp->helpGroup);
		}
	} submit;
	struct Todo : public OptionParser
	{
		Todo(DebtagsOptions* cp) : OptionParser("todo")
		{
			usage = "";
			description = "print a list of the installed packages that are not yet tagged";
			add(&cp->helpGroup);
		}
	} todo;
	struct Score : public OptionParser
	{
		Score(DebtagsOptions* cp) : OptionParser("score")
		{
			usage = "";
			description =
				"score uninstalled packages according to how often their tags "
				"appear in the packages that are installed already";
			add(&cp->helpGroup);
		}
	} score;
	struct Facetcoll : public OptionParser
	{
		Facetcoll(DebtagsOptions* cp) : OptionParser("facetcoll")
		{
			usage = "";
			description =
				"print the tagged collection where each package is tagged with"
				"its facets only.  Useful for further processing via tagcoll";
			add(&cp->outputGroup);
			add(&cp->helpGroup);
		}
	} facetcoll;
	struct Stats : public OptionParser
	{
		Stats(DebtagsOptions* cp) : OptionParser("stats")
		{
			usage = "";
			description = "print statistics about Debtags";
			add(&cp->helpGroup);
		}
	} stats;
	struct Todoreport : public OptionParser
	{
		Todoreport(DebtagsOptions* cp) : OptionParser("todoreport")
		{
			usage = "";
			description = "print a report of packages needing work";
			add(&cp->helpGroup);
		}
	} todoreport;
	struct SmartSearch : public OptionParser
	{
		SmartSearch(DebtagsOptions* cp) : OptionParser("smartsearch")
		{
			usage = "<word [word1 [+tag [-tag1 ...]]]>";
			description =
				"Perform a keyword search integrated with related packages.\n"
				"A + prefix indicates a wanted tag.  A - prefix indicates "
				"an unwanted tag.  Other words indicate keywords to search.\n"
				"Remember to use '--' before unwanted tags to avoid to have "
				"them interpreted as commandline switches.\n";
			add(&cp->helpGroup);
		}
	} smartsearch;

	DebtagsOptions() : MainParser<CommandParser>("debtags"),
		generic(this), help(this), update(this), selfcheck(this), check(this),
		tagcat(this), tagshow(this), tagsearch(this), show(this),
		related(this), cat(this), dumpavail(this), search(this), grep(this), install(this),
		diff(this), maintainers(this), tag(this), submit(this), todo(this),
		score(this), facetcoll(this), stats(this), todoreport(this),
		smartsearch(this)
	{
		add(generic);
		add(help);
		add(update);
		add(selfcheck);
		add(check);
		add(tagcat);
		add(tagshow);
		add(tagsearch);
		add(show);
		add(related);
		add(cat);
		add(dumpavail);
		add(search);
		add(grep);
		add(install);
		add(diff);
		add(maintainers);
		add(tag);
		add(submit);
		add(todo);
		add(score);
		add(facetcoll);
		add(stats);
		add(todoreport);
		add(smartsearch);

		usage = "<command> [options and arguments]";
		description = "Commandline interface to access and manipulate Debian Package Tags";
	}
};

}
}

// vim:set ts=4 sw=4:
#endif
